@@ -9,7 +9,7 @@ from ..enums import TextureFormat, BuildTarget
 TF = TextureFormat
 
 
-def image_to_texture2d(img: Image, target_texture_format: TF, flip: bool = True):
+def image_to_texture2d(img: Image.Image, target_texture_format: TF, flip: bool = True):
     if flip:
         img = img.transpose(Image.FLIP_TOP_BOTTOM)
 
@@ -24,11 +24,13 @@ def image_to_texture2d(img: Image, target_texture_format: TF, flip: bool = True)
         tex_format = TF.DXT5
     # ETC
     elif target_texture_format in [TF.ETC_RGB4, TF.ETC_RGB4Crunched, TF.ETC_RGB4_3DS]:
+        img = assert_rgba(img, target_texture_format)
         r, g, b, a = img.split()
         raw_img = Image.merge("RGBA", (b, g, r, a)).tobytes()
         enc_img = etcpak.compress_to_etc1(raw_img, img.width, img.height)
         tex_format = TF.ETC_RGB4
     elif target_texture_format == TF.ETC2_RGB:
+        img = assert_rgba(img, target_texture_format)
         r, g, b, a = img.split()
         raw_img = Image.merge("RGBA", (b, g, r, a)).tobytes()
         enc_img = etcpak.compress_to_etc2_rgb(raw_img, img.width, img.height)
@@ -37,6 +39,7 @@ def image_to_texture2d(img: Image, target_texture_format: TF, flip: bool = True)
         target_texture_format in [TF.ETC2_RGBA8, TF.ETC2_RGBA8Crunched, TF.ETC2_RGBA1]
         or "_RGB_" in target_texture_format.name
     ):
+        img = assert_rgba(img, target_texture_format)
         r, g, b, a = img.split()
         raw_img = Image.merge("RGBA", (b, g, r, a)).tobytes()
         enc_img = etcpak.compress_to_etc2_rgba(raw_img, img.width, img.height)
@@ -76,7 +79,16 @@ def image_to_texture2d(img: Image, target_texture_format: TF, flip: bool = True)
     return enc_img, tex_format
 
 
-def get_image_from_texture2d(texture_2d, flip=True) -> Image:
+def assert_rgba(img: Image.Image, target_texture_format: TextureFormat):
+    if img.mode == "RGB":
+        img = img.convert("RGBA")
+    assert (
+        img.mode == "RGBA"
+    ), f"{target_texture_format} compression only supports RGB & RGBA images"  # noqa: E501
+    return img
+
+
+def get_image_from_texture2d(texture_2d, flip=True) -> Image.Image:
     """converts the given texture into PIL.Image
 
     :param texture_2d: texture to be converterd
@@ -153,7 +165,7 @@ def pillow(
     codec: str,
     args,
     swap: tuple = None,
-) -> Image:
+) -> Image.Image:
     img = (
         Image.frombytes(mode, (width, height), image_data, codec, args)
         if width
@@ -165,7 +177,7 @@ def pillow(
     return img
 
 
-def atc(image_data: bytes, width: int, height: int, alpha: bool) -> Image:
+def atc(image_data: bytes, width: int, height: int, alpha: bool) -> Image.Image:
     if alpha:
         image_data = texture2ddecoder.decode_atc_rgba8(image_data, width, height)
     else:
@@ -174,7 +186,7 @@ def atc(image_data: bytes, width: int, height: int, alpha: bool) -> Image:
     return Image.frombytes("RGBA", (width, height), image_data, "raw", "BGRA")
 
 
-def astc(image_data: bytes, width: int, height: int, block_size: tuple) -> Image:
+def astc(image_data: bytes, width: int, height: int, block_size: tuple) -> Image.Image:
     image_data = texture2ddecoder.decode_astc(image_data, width, height, *block_size)
     return Image.frombytes("RGBA", (width, height), image_data, "raw", "BGRA")
 
@@ -219,7 +231,7 @@ def half(
     codec: str,
     args,
     swap: tuple = None,
-) -> Image:
+) -> Image.Image:
     # convert half-float to int8
     stream = BytesIO(image_data)
     image_data = bytes(
@@ -254,7 +266,7 @@ CONV_TABLE = {
 (  TF.YUY2,                                                                                     ),
 (  TF.RGB9e5Float,                                                                              ),
 (  TF.BC4,                 pillow,  "L",     "bcn",       4                                     ),
-(  TF.BC5,                 pillow,  "RGBA",  "bcn",       5                                     ),
+(  TF.BC5,                 pillow,  "RGB",   "bcn",       5                                     ),
 (  TF.BC6H,                pillow,  "RGBA",  "bcn",       6                                     ),
 (  TF.BC7,                 pillow,  "RGBA",  "bcn",       7                                     ),
 (  TF.DXT1Crunched,        pillow,  "RGBA",  "bcn",       1                                     ),
