@@ -1,11 +1,18 @@
-﻿from . import File
-from ..helpers import CompressionHelper
+# TODO: implement encryption for saving files
+from collections import namedtuple
+import re
+from typing import Tuple, Union
+
+from . import File
+from ..enums import ArchiveFlags, ArchiveFlagsOld, CompressionFlags
+from ..helpers import ArchiveStorageManager, CompressionHelper
 from ..streams import EndianBinaryReader, EndianBinaryWriter
 
-from collections import namedtuple
+from .. import config
 
 BlockInfo = namedtuple("BlockInfo", "uncompressedSize compressedSize flags")
 DirectoryInfoFS = namedtuple("DirectoryInfoFS", "offset size flags path")
+reVersion = re.compile(r"(\d+)\.(\d+)\.(\d+)\w.+")
 
 
 class BundleFile(File.File):
@@ -14,8 +21,11 @@ class BundleFile(File.File):
     signature: str
     version_engine: str
     version_player: str
+    dataflags: Tuple[ArchiveFlags, ArchiveFlagsOld]
+    decryptor: ArchiveStorageManager.ArchiveStorageDecryptor = None
+    _uses_block_alignment: bool = False
 
-    def __init__(self, reader: EndianBinaryReader, parent: File, name: str=None):
+    def __init__(self, reader: EndianBinaryReader, parent: File, name: str = None):
         super().__init__(parent=parent, name=name)
         signature = self.signature = reader.read_string_to_null()
         self.version = reader.read_u_int()
@@ -23,13 +33,13 @@ class BundleFile(File.File):
         self.version_engine = reader.read_string_to_null()
 
         if signature == "UnityArchive":
-            raise NotImplemented("BundleFile - UnityArchive")
+            raise NotImplementedError("BundleFile - UnityArchive")
         elif signature in ["UnityWeb", "UnityRaw"]:
             m_DirectoryInfo, blocksReader = self.read_web_raw(reader)
         elif signature == "UnityFS":
             m_DirectoryInfo, blocksReader = self.read_fs(reader)
         else:
-            raise NotImplemented(f"Unknown Bundle signature: {signature}")
+            raise NotImplementedError(f"Unknown Bundle signature: {signature}")
 
         self.read_files(blocksReader, m_DirectoryInfo)
 
@@ -75,36 +85,59 @@ class BundleFile(File.File):
         return m_DirectoryInfo, blocksReader
 
     def read_fs(self, reader: EndianBinaryReader):
-        version = self.version
         size = reader.read_long()
 
         # header
         compressedSize = reader.read_u_int()
         uncompressedSize = reader.read_u_int()
-        self._data_flags = reader.read_u_int()
+        self.dataflags = reader.read_u_int()
 
+        version = self.get_version_tuple()
+        # https://issuetracker.unity3d.com/issues/files-within-assetbundles-do-not-start-on-aligned-boundaries-breaking-patching-on-nintendo-switch
+        # Unity CN introduced encryption before the alignment fix was introduced.
+        # Unity CN used the same flag for the encryption as later on the alignment fix,
+        # so we have to check the version to determine the correct flag set.
+        if (
+            version < (2020,)
+            or (version[0] == 2020 and version < (2020, 3, 34))
+            or (version[0] == 2021 and version < (2021, 3, 2))
+            or (version[0] == 2022 and version < (2022, 1, 1))
+        ):
+            self.dataflags = ArchiveFlagsOld(self.dataflags)
+        else:
+            self.dataflags = ArchiveFlags(self.dataflags)
+
+        if self.dataflags & self.dataflags.UsesAssetBundleEncryption:
+            self.decryptor = ArchiveStorageManager.ArchiveStorageDecryptor(reader)
+
+        # check if we need to align the reader
+        # - align to 16 bytes and check if all are 0
+        # - if not, reset the reader to the previous position
         if self.version >= 7:
             reader.align_stream(16)
+            self._uses_block_alignment = True
+        elif version >= (2019, 4):
+            pre_align = reader.Position
+            align_data = reader.read((16 - pre_align % 16) % 16)
+            if any(align_data):
+                reader.Position = pre_align
+            else:
+                self._uses_block_alignment = True
 
-        _position = reader.Position
-        if self._data_flags & 0x80 != 0:  # kArchiveBlocksInfoAtTheEnd
+        start = reader.Position
+        if (
+            self.dataflags & ArchiveFlags.BlocksInfoAtTheEnd
+        ):  # kArchiveBlocksInfoAtTheEnd
             reader.Position = reader.Length - compressedSize
             blocksInfoBytes = reader.read_bytes(compressedSize)
-            reader.Position = _position
+            reader.Position = start
         else:  # 0x40 kArchiveBlocksAndDirectoryInfoCombined
             blocksInfoBytes = reader.read_bytes(compressedSize)
 
-        switch = self._data_flags & 0x3F
-
-        if switch == 1:  # LZMA
-            blocksInfoBytes = CompressionHelper.decompress_lzma(blocksInfoBytes)
-        elif switch in [2, 3]:  # LZ4, LZ4HC
-            blocksInfoBytes = CompressionHelper.decompress_lz4(
-                blocksInfoBytes, uncompressedSize
-            )
-        # elif switch == 4: #LZHAM:
-
-        blocksInfoReader = EndianBinaryReader(blocksInfoBytes, offset=_position)
+        blocksInfoBytes = self.decompress_data(
+            blocksInfoBytes, uncompressedSize, self.dataflags
+        )
+        blocksInfoReader = EndianBinaryReader(blocksInfoBytes, offset=start)
 
         uncompressedDataHash = blocksInfoReader.read_bytes(16)
         blocksInfoCount = blocksInfoReader.read_int()
@@ -129,44 +162,41 @@ class BundleFile(File.File):
             for _ in range(nodesCount)
         ]
 
-        # def read_blocks(reader : EndianBinaryReader, blocksStream):
-        def decompress_block(blockInfo):
-            switch = blockInfo.flags & 0x3F  # kStorageBlockCompressionTypeMask
-            if switch == 1:  # LZMA
-                return CompressionHelper.decompress_lzma(
-                    reader.read_bytes(blockInfo.compressedSize)
-                )
-            elif switch in [2, 3]:  # LZ4, LZ4HC
-                return CompressionHelper.decompress_lz4(
-                    reader.read_bytes(blockInfo.compressedSize),
-                    blockInfo.uncompressedSize,
-                )
-            # elif switch == 4: #LZHAM:
-            else:  # no compression
-                return reader.read_bytes(blockInfo.uncompressedSize)
-
         if m_BlocksInfo:
             self._block_info_flags = m_BlocksInfo[0].flags
 
+        if (
+            isinstance(self.dataflags, ArchiveFlags)
+            and self.dataflags & ArchiveFlags.BlockInfoNeedPaddingAtStart
+        ):
+            reader.align_stream(16)
+
         blocksReader = EndianBinaryReader(
-            b"".join(decompress_block(blockInfo) for blockInfo in m_BlocksInfo),
-            offset=(blocksInfoReader.real_offset())
+            b"".join(
+                self.decompress_data(
+                    reader.read_bytes(blockInfo.compressedSize),
+                    blockInfo.uncompressedSize,
+                    blockInfo.flags,
+                    i,
+                )
+                for i, blockInfo in enumerate(m_BlocksInfo)
+            ),
+            offset=(blocksInfoReader.real_offset()),
         )
 
         return m_DirectoryInfo, blocksReader
 
-
     def save(self, packer=None):
         """
-            Rewrites the BundleFile and returns it as bytes object.
+        Rewrites the BundleFile and returns it as bytes object.
 
-            packer:
-                can be either one of the following strings
-                or tuple consisting of (block_info_flag, data_flag)
-                allowed strings:
-                    none - no compression, default, safest bet
-                    lz4 - lz4 compression
-                    original - uses the original flags
+        packer:
+            can be either one of the following strings
+            or tuple consisting of (block_info_flag, data_flag)
+            allowed strings:
+                none - no compression, default, safest bet
+                lz4 - lz4 compression
+                original - uses the original flags
         """
         # file_header
         #     signature    (string_to_null)
@@ -181,24 +211,30 @@ class BundleFile(File.File):
         writer.write_string_to_null(self.version_engine)
 
         if self.signature == "UnityArchive":
-            raise NotImplemented("BundleFile - UnityArchive")
+            raise NotImplementedError("BundleFile - UnityArchive")
         elif self.signature in ["UnityWeb", "UnityRaw"]:
-            raise NotImplemented("Saving Unity Web and Raw bundles isn't supported yet")
+            raise NotImplementedError(
+                "Saving Unity Web and Raw bundles isn't supported yet"
+            )
             # self.save_web_raw(writer)
         elif self.signature == "UnityFS":
             if not packer or packer == "none":
                 self.save_fs(writer, 64, 64)
             elif packer == "original":
-                self.save_fs(writer, block_info_flag=self._block_info_flags, data_flag=self._data_flags)
+                self.save_fs(
+                    writer,
+                    data_flag=self._data_flags,
+                    block_info_flag=self._block_info_flags,
+                )
             elif packer == "lz4":
-                self.save_fs(writer, block_info_flag=194, data_flag=2)
+                self.save_fs(writer, data_flag=194, block_info_flag=2)
             elif isinstance(packer, tuple):
                 self.save_fs(writer, *packer)
             else:
-                raise NotImplemented("UnityFS - Packer:", packer)
+                raise NotImplementedError("UnityFS - Packer:", packer)
         return writer.bytes
 
-    def save_fs(self, writer: EndianBinaryWriter, block_info_flag : int, data_flag : int):
+    def save_fs(self, writer: EndianBinaryWriter, data_flag: int, block_info_flag: int):
         # header
         # compressed blockinfo (block details & directionary)
         # compressed assets
@@ -254,7 +290,9 @@ class BundleFile(File.File):
                 name,
                 f.flags,
                 data_writer.write_bytes(
-                    f.bytes if isinstance(f, (EndianBinaryReader, EndianBinaryWriter)) else f.save()
+                    f.bytes
+                    if isinstance(f, (EndianBinaryReader, EndianBinaryWriter))
+                    else f.save()
                 ),
             )
             for name, f in self.files.items()
@@ -265,7 +303,7 @@ class BundleFile(File.File):
         uncompressed_data_size = len(file_data)
 
         # compress the data
-        switch = data_flag & 0x3F
+        switch = block_info_flag & 0x3F
         if switch == 1:  # LZMA
             file_data = CompressionHelper.compress_lzma(file_data)
         elif switch in [2, 3]:  # LZ4, LZ4HC
@@ -286,9 +324,13 @@ class BundleFile(File.File):
         # compressed size
         block_writer.write_u_int(compressed_data_size)
         # flag
-        block_writer.write_u_short(data_flag)
+        block_writer.write_u_short(block_info_flag)
 
         # file block info
+        if not data_flag & 0x40:
+            raise NotImplementedError(
+                "UnityPy always writes DirectoryInfo, so data_flag must include 0x40"
+            )
         # file count
         block_writer.write_int(len(files))
         offset = 0
@@ -309,7 +351,7 @@ class BundleFile(File.File):
 
         uncompressed_block_data_size = len(block_data)
 
-        switch = block_info_flag & 0x3F
+        switch = data_flag & 0x3F
         if switch == 1:  # LZMA
             block_data = CompressionHelper.compress_lzma(block_data)
         elif switch in [2, 3]:  # LZ4, LZ4HC
@@ -320,25 +362,17 @@ class BundleFile(File.File):
         compressed_block_data_size = len(block_data)
 
         # write the header info
-        ## file size
-        writer.write_long(
-            writer.Length
-            + 8
-            + 4
-            + 4
-            + 4
-            + (8 if self.version >= 7 else 0)
-            + compressed_block_data_size
-            + compressed_data_size
-        )
+        ## file size - 0 for now, will be set at the end
+        writer_header_pos = writer.Position
+        writer.write_long(0)
         # compressed blockInfoBytes size
         writer.write_u_int(compressed_block_data_size)
         # uncompressed size
         writer.write_u_int(uncompressed_block_data_size)
-        # compression flag
-        writer.write_u_int(block_info_flag)
+        # compression and file layout flag
+        writer.write_u_int(data_flag)
 
-        if self.version >= 7:
+        if self._uses_block_alignment:
             # UnityFS\x00 - 8
             # size 8
             # comp sizes 4+4
@@ -346,9 +380,60 @@ class BundleFile(File.File):
             # sum : 28 -> +8 alignment
             writer.align_stream(16)
 
-        if (block_info_flag & 0x80) != 0:  # at end of file
+        if data_flag & 0x80:  # at end of file
+            if data_flag & 0x200:
+                writer.align_stream(16)
             writer.write(file_data)
             writer.write(block_data)
         else:
             writer.write(block_data)
+            if data_flag & 0x200:
+                writer.align_stream(16)
             writer.write(file_data)
+
+        writer_end_pos = writer.Position
+        writer.Position = writer_header_pos
+        # correct file size
+        writer.write_long(writer_end_pos)
+        writer.Position = writer_end_pos
+
+    def decompress_data(
+        self,
+        compressed_data: bytes,
+        uncompressed_size: int,
+        flags: Union[int, ArchiveFlags, ArchiveFlagsOld],
+        index: int = 0,
+    ) -> bytes:
+        """
+        Parameters
+        ----------
+        compressed_data : bytes
+            The compressed data.
+        uncompressed_size : int
+            The uncompressed size of the data.
+        flags : int
+            The flags of the data.
+
+        Returns
+        -------
+        bytes
+            The decompressed data."""
+        comp_flag = CompressionFlags(flags & ArchiveFlags.CompressionTypeMask)
+
+        if comp_flag == CompressionFlags.LZMA:  # LZMA
+            return CompressionHelper.decompress_lzma(compressed_data)
+        elif comp_flag in [CompressionFlags.LZ4, CompressionFlags.LZ4HC]:  # LZ4, LZ4HC
+            if self.decryptor is not None and flags & 0x100:
+                compressed_data = self.decryptor.decrypt_block(compressed_data, index)
+            return CompressionHelper.decompress_lz4(compressed_data, uncompressed_size)
+        elif comp_flag == CompressionFlags.LZHAM:  # LZHAM
+            raise NotImplementedError("LZHAM decompression not implemented")
+        else:
+            return compressed_data
+
+    def get_version_tuple(self) -> Tuple[int, int, int]:
+        """Returns the version as a tuple."""
+        version = self.version_engine
+        if not version or version == "0.0.0":
+            version = config.get_fallback_version()
+        return tuple(map(int, reVersion.match(version).groups()))
